@@ -31,15 +31,17 @@ import logging
 from enum import Enum, unique
 
 import nbxmpp
+from nbxmpp.namespaces import Namespace
+from nbxmpp.util import generate_id
 
 from gajim.common import app
-from gajim.common.jingle_transport import get_jingle_transport, JingleTransportIBB
-from gajim.common.jingle_content import get_jingle_content, JingleContentSetupException
+from gajim.common.jingle_transport import get_jingle_transport
+from gajim.common.jingle_transport import JingleTransportIBB
+from gajim.common.jingle_content import get_jingle_content
+from gajim.common.jingle_content import JingleContentSetupException
 from gajim.common.jingle_ft import State
-from gajim.common.connection_handlers_events import (
-    FilesProp, JingleRequestReceivedEvent, JingleDisconnectedReceivedEvent,
-    JingleTransferCancelledEvent, JingleConnectedReceivedEvent,
-    JingleErrorReceivedEvent)
+from gajim.common.connection_handlers_events import FilesProp
+from gajim.common.nec import NetworkEvent
 
 log = logging.getLogger("app.c.jingle_session")
 
@@ -87,9 +89,7 @@ class JingleSession:
         self.contents = {} # negotiated contents
         self.connection = con # connection to use
         # our full jid
-        self.ourjid = app.get_jid_from_account(self.connection.name)
-        if con.server_resource:
-            self.ourjid = self.ourjid + '/' + con.server_resource
+        self.ourjid = str(self.connection.get_own_jid())
         self.peerjid = jid # jid we connect to
         # jid we use as the initiator
         self.initiator = self.ourjid if weinitiate else self.peerjid
@@ -103,7 +103,7 @@ class JingleSession:
         # what state is session in? (one from JingleStates)
         self.state = JingleStates.ENDED
         if not sid:
-            sid = con.connection.getAnID()
+            sid = generate_id()
         self.sid = sid # sessionid
         # iq stanza id, used to determine which sessions to summon callback
         # later on when iq-result stanza arrives
@@ -140,7 +140,9 @@ class JingleSession:
             'transport-info':       [self.__ack, self.__broadcast],
             'transport-replace':    [self.__ack, self.__broadcast,
                                      self.__on_transport_replace], #TODO
-            'transport-accept':     [self.__ack], #TODO
+            'transport-accept':     [self.__ack, self.__on_session_accept,
+                                     self.__on_content_accept,
+                                     self.__broadcast],
             'transport-reject':     [self.__ack], #TODO
             'iq-result':            [self.__broadcast],
             'iq-error':             [self.__on_error],
@@ -244,8 +246,9 @@ class JingleSession:
         Currently used for transport replacement
         '''
         content = self.contents[(creator, name)]
-        transport.set_sid(content.transport.sid)
-        transport.set_file_props(content.transport.file_props)
+        file_props = content.transport.file_props
+        file_props.transport_sid = transport.sid
+        transport.set_file_props(file_props)
         content.transport = transport
         # The content will have to be resend now that it is modified
         content.sent = False
@@ -377,10 +380,10 @@ class JingleSession:
         text = error.getTagData('text')
         error_name = None
         for child in error.getChildren():
-            if child.getNamespace() == nbxmpp.NS_JINGLE_ERRORS:
+            if child.getNamespace() == Namespace.JINGLE_ERRORS:
                 error_name = child.getName()
                 break
-            elif child.getNamespace() == nbxmpp.NS_STANZAS:
+            if child.getNamespace() == Namespace.STANZAS:
                 error_name = child.getName()
         self.__dispatch_error(error_name, text, error.getAttr('type'))
 
@@ -404,18 +407,19 @@ class JingleSession:
             name = content['name']
             if (creator, name) in self.contents:
                 transport_ns = content.getTag('transport').getNamespace()
-                if transport_ns == nbxmpp.NS_JINGLE_ICE_UDP:
+                if transport_ns == Namespace.JINGLE_ICE_UDP:
                     # FIXME: We don't manage anything else than ICE-UDP now...
                     # What was the previous transport?!?
                     # Anyway, content's transport is not modifiable yet
                     pass
-                elif transport_ns == nbxmpp.NS_JINGLE_IBB:
-                    transport = JingleTransportIBB()
+                elif transport_ns == Namespace.JINGLE_IBB:
+                    transport = JingleTransportIBB(node=content.getTag(
+                        'transport'))
                     self.modify_content(creator, name, transport)
                     self.state = JingleStates.PENDING
                     self.contents[(creator, name)].state = State.TRANSPORT_REPLACE
                     self.__ack(stanza, jingle, error, action)
-                    self.__session_accept()
+                    self.__transport_accept(transport)
                 else:
                     stanza, jingle = self.__make_jingle('transport-reject')
                     content = jingle.setTag('content', attrs={'creator': creator,
@@ -444,7 +448,7 @@ class JingleSession:
         for child in payload:
             if child.getName() == 'checksum':
                 hash_ = child.getTag('file').getTag(name='hash',
-                                                    namespace=nbxmpp.NS_HASHES_2)
+                                                    namespace=Namespace.HASHES_2)
                 if hash_ is None:
                     continue
                 algo = hash_.getAttr('algo')
@@ -465,9 +469,9 @@ class JingleSession:
             if (creator, name) in self.contents:
                 content = self.contents[(creator, name)]
                 # TODO: this will fail if content is not an RTP content
-                app.nec.push_incoming_event(JingleDisconnectedReceivedEvent(
-                    None, conn=self.connection, jingle_session=self,
-                    media=content.media, reason='removed'))
+                self._raise_event('jingle-disconnected-received',
+                                  media=content.media,
+                                  reason='removed')
                 content.destroy()
         if not self.contents:
             reason = nbxmpp.Node('reason')
@@ -503,10 +507,7 @@ class JingleSession:
         #     self.add_content(name, content, creator)
         #     self.__content_reject(content)
         #     self.contents[(content.creator, content.name)].destroy()
-        app.nec.push_incoming_event(JingleRequestReceivedEvent(None,
-                                                                 conn=self.connection,
-                                                                 jingle_session=self,
-                                                                 contents=contents))
+        self._raise_event('jingle-request-received', contents=contents)
 
     def __on_session_initiate(self, stanza, jingle, error, action):
         """
@@ -538,7 +539,7 @@ class JingleSession:
         # If we are not receiving a file
         # Check if there's already a session with this user:
         if contents[0].media != 'file':
-            for session in self.connection.iter_jingle_sessions(self.peerjid):
+            for session in self.connection.get_module('Jingle').get_jingle_sessions(self.peerjid):
                 if session is not self:
                     reason = nbxmpp.Node('reason')
                     alternative_session = reason.setTag('alternative-session')
@@ -557,21 +558,18 @@ class JingleSession:
                 n = request.getTag('file').getTag('name')
                 n = n.getData() if n else None
                 pjid = app.get_jid_without_resource(self.peerjid)
-                file_info = self.connection.get_file_info(pjid, hash_data, n,
-                                                          self.connection.name)
+                file_info = self.connection.get_module('Jingle').get_file_info(
+                    pjid, hash_data, n, self.connection.name)
                 if not file_info:
                     log.warning('The peer %s is requesting a ' \
-                                'file that we dont have or ' \
+                                'file that we don’t have or ' \
                                 'it is not allowed to request', pjid)
                     self.decline_session()
                     raise nbxmpp.NodeProcessed
 
         self.state = JingleStates.PENDING
         # Send event about starting a session
-        app.nec.push_incoming_event(JingleRequestReceivedEvent(None,
-                                                                 conn=self.connection,
-                                                                 jingle_session=self,
-                                                                 contents=contents[0]))
+        self._raise_event('jingle-request-received', contents=contents[0])
 
     def __broadcast(self, stanza, jingle, error, action):
         """
@@ -591,14 +589,14 @@ class JingleSession:
             creator = content['creator']
             if (creator, name) not in self.contents:
                 text = 'Content %s (created by %s) does not exist' % (name, creator)
-                self.__send_error(stanza, 'bad-request', text=text, type_='_modify')
+                self.__send_error(stanza, 'bad-request', text=text, type_='modify')
                 raise nbxmpp.NodeProcessed
-            else:
-                cn = self.contents[(creator, name)]
-                cn.on_stanza(stanza, content, error, action)
+
+            cn = self.contents[(creator, name)]
+            cn.on_stanza(stanza, content, error, action)
 
     def __on_session_terminate(self, stanza, jingle, error, action):
-        self.connection.delete_jingle_session(self.sid)
+        self.connection.get_module('Jingle').delete_jingle_session(self.sid)
         reason, text = self.__reason_from_stanza(jingle)
         if reason not in ('success', 'cancel', 'decline'):
             self.__dispatch_error(reason, text)
@@ -608,11 +606,9 @@ class JingleSession:
             # TODO
             text = reason
         if reason == 'cancel' and self.session_type_ft:
-            app.nec.push_incoming_event(JingleTransferCancelledEvent(None,
-                                                                       conn=self.connection,
-                                                                       jingle_session=self,
-                                                                       media=None,
-                                                                       reason=text))
+            self._raise_event('jingle-ft-cancelled-received',
+                              media=None,
+                              reason=text)
 
     def __broadcast_all(self, stanza, jingle, error, action):
         """
@@ -659,10 +655,7 @@ class JingleSession:
         if text:
             text = '%s (%s)' % (error, text)
         if type_ != 'modify':
-            app.nec.push_incoming_event(JingleErrorReceivedEvent(None,
-                                                                   conn=self.connection,
-                                                                   jingle_session=self,
-                                                                   reason=text or error))
+            self._raise_event('jingle-error-received', reason=text or error)
 
     @staticmethod
     def __reason_from_stanza(stanza):
@@ -694,18 +687,18 @@ class JingleSession:
             'initiator' : self.initiator
         }
         jingle = stanza.addChild('jingle', attrs=attrs,
-                                 namespace=nbxmpp.NS_JINGLE)
+                                 namespace=Namespace.JINGLE)
         if reason is not None:
             jingle.addChild(node=reason)
         return stanza, jingle
 
     def __send_error(self, stanza, error, jingle_error=None, text=None, type_=None):
-        err_stanza = nbxmpp.Error(stanza, '%s %s' % (nbxmpp.NS_STANZAS, error))
+        err_stanza = nbxmpp.Error(stanza, '%s %s' % (Namespace.STANZAS, error))
         err = err_stanza.getTag('error')
         if type_:
             err.setAttr('type', type_)
         if jingle_error:
-            err.setTag(jingle_error, namespace=nbxmpp.NS_JINGLE_ERRORS)
+            err.setTag(jingle_error, namespace=Namespace.JINGLE_ERRORS)
         if text:
             err.setTagData('text', text)
         self.connection.connection.send(err_stanza)
@@ -766,7 +759,7 @@ class JingleSession:
     def _session_terminate(self, reason=None):
         stanza, jingle = self.__make_jingle('session-terminate', reason=reason)
         self.__broadcast_all(stanza, jingle, None, 'session-terminate-sent')
-        if self.connection.connection and self.connection.connected >= 2:
+        if self.connection.connection and self.connection.state.is_available:
             self.connection.connection.send(stanza)
         # TODO: Move to GUI?
         reason, text = self.__reason_from_stanza(jingle)
@@ -776,12 +769,10 @@ class JingleSession:
             text = '%s (%s)' % (reason, text)
         else:
             text = reason
-        self.connection.delete_jingle_session(self.sid)
-        app.nec.push_incoming_event(JingleDisconnectedReceivedEvent(None,
-                                                                      conn=self.connection,
-                                                                      jingle_session=self,
-                                                                      media=None,
-                                                                      reason=text))
+        self.connection.get_module('Jingle').delete_jingle_session(self.sid)
+        self._raise_event('jingle-disconnected-received',
+                          media=None,
+                          reason=text)
 
     def __content_add(self, content):
         # TODO: test
@@ -807,30 +798,44 @@ class JingleSession:
         self.__append_content(jingle, content)
         self.connection.connection.send(stanza)
         # TODO: this will fail if content is not an RTP content
-        app.nec.push_incoming_event(JingleDisconnectedReceivedEvent(None,
-                                                                      conn=self.connection,
-                                                                      jingle_session=self,
-                                                                      media=content.media,
-                                                                      reason='rejected'))
+        self._raise_event('jingle-disconnected-received',
+                          media=content.media,
+                          reason='rejected')
 
     def __content_modify(self):
         assert self.state != JingleStates.ENDED
 
     def __content_remove(self, content, reason=None):
         assert self.state != JingleStates.ENDED
-        if self.connection.connection and self.connection.connected > 1:
+        if self.connection.connection and self.connection.state.is_available:
             stanza, jingle = self.__make_jingle('content-remove', reason=reason)
             self.__append_content(jingle, content)
             self.connection.connection.send(stanza)
         # TODO: this will fail if content is not an RTP content
-        app.nec.push_incoming_event(JingleDisconnectedReceivedEvent(None,
-                                                                      conn=self.connection,
-                                                                      jingle_session=self,
-                                                                      media=content.media,
-                                                                      reason='removed'))
+        self._raise_event('jingle-disconnected-received',
+                          media=content.media,
+                          reason='removed')
 
     def content_negotiated(self, media):
-        app.nec.push_incoming_event(JingleConnectedReceivedEvent(None,
-                                                                   conn=self.connection,
-                                                                   jingle_session=self,
-                                                                   media=media))
+        self._raise_event('jingle-connected-received', media=media)
+
+    def __transport_accept(self, transport):
+        assert self.state != JingleStates.ENDED
+        stanza, jingle = self.__make_jingle('transport-accept')
+        self.__append_contents(jingle)
+        self.__broadcast(stanza, jingle, None, 'transport-accept')
+        self.connection.connection.send(stanza)
+        self.collect_iq_id(stanza.getID())
+        self.state = JingleStates.ACTIVE
+
+    def _raise_event(self, name, **kwargs):
+        jid, resource = app.get_room_and_nick_from_fjid(self.peerjid)
+        app.nec.push_incoming_event(
+            NetworkEvent(name,
+                         conn=self.connection,
+                         fjid=self.peerjid,
+                         jid=jid,
+                         sid=self.sid,
+                         resource=resource,
+                         jingle_session=self,
+                         **kwargs))
