@@ -15,29 +15,29 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim.  If not, see <http://www.gnu.org/licenses/>.
 
-from gajim.common import app
-import nbxmpp
-from nbxmpp.idlequeue import IdleObject
-from nbxmpp import dispatcher_nb, simplexml
-from nbxmpp.plugin import PlugIn
-from nbxmpp.plugin import *
-from nbxmpp.transports_nb import DATA_RECEIVED, DATA_SENT, DATA_ERROR
-from gajim.common.zeroconf import zeroconf
-
-from nbxmpp.protocol import *
 import socket
-import platform
 import ssl
 import errno
 import sys
 import os
-import string
-from random import Random
-
 import logging
+from unittest.mock import Mock
+
+import nbxmpp
+from nbxmpp import old_dispatcher as dispatcher
+from nbxmpp import simplexml
+from nbxmpp.namespaces import Namespace
+from nbxmpp.structs import StanzaHandler
+from nbxmpp.plugin import PlugIn
+from nbxmpp.idlequeue import IdleObject
+from nbxmpp.util import generate_id
+
+from gajim.common import app
+from gajim.common.zeroconf import zeroconf
+from gajim.common.zeroconf import roster_zeroconf
+
 log = logging.getLogger('gajim.c.z.client_zeroconf')
 
-from gajim.common.zeroconf import roster_zeroconf
 
 MAX_BUFF_LEN = 65536
 TYPE_SERVER, TYPE_CLIENT = range(2)
@@ -51,7 +51,7 @@ ACTIVITY_TIMEOUT_SECONDS = 30
 class ZeroconfListener(IdleObject):
     def __init__(self, port, conn_holder):
         """
-        Handle all incomming connections on ('0.0.0.0', port)
+        Handle all incoming connections on ('0.0.0.0', port)
         """
         self.port = port
         self.queue_idx = -1
@@ -73,7 +73,7 @@ class ZeroconfListener(IdleObject):
         self._serv.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
         self._serv.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         if os.name == 'nt':
-            if int(platform.win32_ver()[0]) >= 6:  # Win Vista +
+            if sys.getwindowsversion().major >= 6:  # Win Vista +
                 # 47 is socket.IPPROTO_IPV6
                 # 27 is socket.IPV6_V6ONLY under windows, but not defined ...
                 self._serv.setsockopt(41, 27, 0)
@@ -97,7 +97,7 @@ class ZeroconfListener(IdleObject):
 
     def pollin(self):
         """
-        Accept a new incomming connection and notify queue
+        Accept a new incoming connection and notify queue
         """
         sock = self.accept_conn()
         # loop through roster to find who has connected to us
@@ -141,6 +141,7 @@ class P2PClient(IdleObject):
         self.Namespace = 'jabber:client'
         self.protocol_type = 'XMPP'
         self.defaultNamespace = self.Namespace
+        self.Smacks = Mock()
         self._component = 0
         self._registered_name = None
         self._caller = conn_holder.caller
@@ -161,7 +162,7 @@ class P2PClient(IdleObject):
             self.on_connect, self)
         self.Server = conn.host  # set Server to the last host name / address tried
         if not self.conn_holder:
-            # An error occured, disconnect() has been called
+            # An error occurred, disconnect() has been called
             if on_not_ok:
                 on_not_ok('Connection to host could not be established.')
             return
@@ -180,7 +181,7 @@ class P2PClient(IdleObject):
                 thread_id = stanza.getThread()
                 id_ = stanza.getID()
                 if not id_:
-                    id_ = self.Dispatcher.getAnID()
+                    id_ = generate_id()
                 if self.fd in self.conn_holder.ids_of_awaiting_messages:
                     self.conn_holder.ids_of_awaiting_messages[self.fd].append((
                         id_, thread_id))
@@ -190,6 +191,9 @@ class P2PClient(IdleObject):
 
         self.on_responses = {}
 
+    def get_bound_jid(self):
+        return self._caller.get_own_jid()
+
     def add_stanza(self, stanza, is_message=False):
         if self.Connection:
             if self.Connection.state == -1:
@@ -202,7 +206,7 @@ class P2PClient(IdleObject):
             thread_id = stanza.getThread()
             id_ = stanza.getID()
             if not id_:
-                id_ = self.Dispatcher.getAnID()
+                id_ = generate_id()
             if self.fd in self.conn_holder.ids_of_awaiting_messages:
                 self.conn_holder.ids_of_awaiting_messages[self.fd].append((id_,
                     thread_id))
@@ -224,7 +228,7 @@ class P2PClient(IdleObject):
     def on_connect(self, conn):
         self.Connection = conn
         self.Connection.PlugIn(self)
-        dispatcher_nb.Dispatcher().PlugIn(self)
+        dispatcher.Dispatcher().PlugIn(self)
         self._register_handlers()
 
     def StreamInit(self):
@@ -243,7 +247,7 @@ class P2PClient(IdleObject):
         self.Dispatcher._metastream = nbxmpp.Node('stream:stream')
         self.Dispatcher._metastream.setNamespace(self.Namespace)
         self.Dispatcher._metastream.setAttr('version', '1.0')
-        self.Dispatcher._metastream.setAttr('xmlns:stream', nbxmpp.NS_STREAMS)
+        self.Dispatcher._metastream.setAttr('xmlns:stream', Namespace.STREAMS)
         self.Dispatcher._metastream.setAttr('from',
             self.conn_holder.zeroconf.name)
         if self.to:
@@ -252,7 +256,7 @@ class P2PClient(IdleObject):
                 self.Dispatcher._metastream)[:-2])
 
     def _check_stream_start(self, ns, tag, attrs):
-        if ns != nbxmpp.NS_STREAMS or tag != 'stream':
+        if ns != Namespace.STREAMS or tag != 'stream':
             log.error('Incorrect stream start: (%s,%s).Terminating!',
                       tag, ns)
             self.Connection.disconnect()
@@ -282,6 +286,7 @@ class P2PClient(IdleObject):
                 del self.conn_holder.ids_of_awaiting_messages[self.fd]
             self.conn_holder.remove_connection(self.sock_hash)
         if 'Dispatcher' in self.__dict__:
+            self._caller._unregister_new_handlers(self)
             self.Dispatcher.PlugOut()
         if 'P2PConnection' in self.__dict__:
             self.P2PConnection.PlugOut()
@@ -315,23 +320,15 @@ class P2PClient(IdleObject):
 
     def _register_handlers(self):
         self._caller.peerhost = self.Connection._sock.getsockname()
-        self.RegisterHandler('message', lambda conn,
-            data: self._caller._messageCB(self.Server, conn, data))
-        self.RegisterHandler('iq', self._caller._siSetCB, 'set', nbxmpp.NS_SI)
-        self.RegisterHandler('iq', self._caller._siErrorCB, 'error',
-            nbxmpp.NS_SI)
-        self.RegisterHandler('iq', self._caller._siResultCB, 'result',
-            nbxmpp.NS_SI)
-        self.RegisterHandler('iq', self._caller._bytestreamSetCB, 'set',
-            nbxmpp.NS_BYTESTREAM)
-        self.RegisterHandler('iq', self._caller._bytestreamResultCB, 'result',
-            nbxmpp.NS_BYTESTREAM)
-        self.RegisterHandler('iq', self._caller._bytestreamErrorCB, 'error',
-            nbxmpp.NS_BYTESTREAM)
-        self.RegisterHandler('iq', self._caller._JingleCB, 'result')
-        self.RegisterHandler('iq', self._caller._JingleCB, 'error')
-        self.RegisterHandler('iq', self._caller._JingleCB, 'set',
-            nbxmpp.NS_JINGLE)
+
+        self.RegisterHandler(*StanzaHandler(name='message',
+                                            callback=self._caller._messageCB))
+        self.RegisterHandler(*StanzaHandler(name='message',
+                                            typ='error',
+                                            callback=self._caller._message_error_received))
+
+        self._caller._register_new_handlers(self)
+
 
 class P2PConnection(IdleObject, PlugIn):
     def __init__(self, sock_hash, _sock, addresses=None, caller=None,
@@ -429,7 +426,7 @@ class P2PConnection(IdleObject, PlugIn):
                 self.on_receive = None
             return
         _tmp = self.on_receive
-        # make sure this cb is not overriden by recursive calls
+        # make sure this cb is not overridden by recursive calls
         if not recv_handler(None) and _tmp == self.on_receive:
             self.on_receive = recv_handler
 
@@ -455,7 +452,7 @@ class P2PConnection(IdleObject, PlugIn):
         if self.fd in ids and ids[self.fd]:
             for (_id, thread_id) in ids[self.fd]:
                 if hasattr(self._owner, 'Dispatcher'):
-                    self._owner.Dispatcher.Event('', DATA_ERROR, (
+                    self._owner.Dispatcher.Event('', 'DATA ERROR', (
                         self.client.to, thread_id))
                 else:
                     self._owner.on_not_ok('connection timeout')
@@ -521,7 +518,7 @@ class P2PConnection(IdleObject, PlugIn):
             pass
         elif errnum in [errno.ECONNRESET, errno.ENOTCONN, errno.ESHUTDOWN]:
             self.pollend()
-            # don't proccess result, cas it will raise error
+            # don’t process result, in case it will raise an error
             return
         elif not received:
             if errnum != ssl.SSL_ERROR_EOF:
@@ -542,7 +539,7 @@ class P2PConnection(IdleObject, PlugIn):
             if received.strip():
                 log.debug('received: %s', received)
             if hasattr(self._owner, 'Dispatcher'):
-                self._owner.Dispatcher.Event('', DATA_RECEIVED, received)
+                self._owner.Dispatcher.Event('', 'DATA RECEIVED', received)
             self.on_receive(received)
         else:
             # This should never happed, so we need the debug
@@ -611,7 +608,7 @@ class P2PConnection(IdleObject, PlugIn):
             log.debug('sent: %s', self.sent_data)
             if hasattr(self._owner, 'Dispatcher'):
                 self._owner.Dispatcher.Event(
-                    '', DATA_SENT, self.sent_data.decode('utf-8'))
+                    '', 'DATA SENT', self.sent_data.decode('utf-8'))
         self.sent_data = None
         if self.buff_is_message:
             self._owner.on_message_sent(self.fd)
@@ -676,7 +673,7 @@ class ClientZeroconf:
     def zeroconf_init(self, show, msg):
         self.zeroconf = zeroconf.Zeroconf(self.caller._on_new_service,
             self.caller._on_remove_service, self.caller._on_name_conflictCB,
-            self.caller._on_disconnected, self.caller._on_error,
+            self.caller._on_disconnect, self.caller._on_error,
             self.caller.username, self.caller.host, self.port)
         self.zeroconf.txt['msg'] = msg
         self.zeroconf.txt['status'] = show
@@ -800,12 +797,6 @@ class ClientZeroconf:
         P2PClient(None, addresses_, self,
             [(stanza, is_message)], to, on_ok=on_ok, on_not_ok=on_not_ok)
 
-    def getAnID(self):
-        """
-        Generate a random id
-        """
-        return ''.join(Random().sample(string.ascii_letters + string.digits, 6))
-
     def RegisterDisconnectHandler(self, handler):
         """
         Register handler that will be called on disconnect
