@@ -23,28 +23,47 @@
 # You should have received a copy of the GNU General Public License
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
-import time
+import logging
 
 from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GObject
 from gi.repository import GLib
+from gi.repository import Gio
 
-from gajim import common
 from gajim.common import app
+from gajim.common import ged
 from gajim.common.i18n import Q_
 from gajim.common.i18n import _
+from gajim.common.nec import EventHelper
 
 from gajim import gtkgui_helpers
-from gajim import message_control
 from gajim.chat_control_base import ChatControlBase
 from gajim.chat_control import ChatControl
 
-from gajim.gtk.dialogs import YesNoDialog
+from gajim.gtk.dialogs import DialogButton
+from gajim.gtk.dialogs import NewConfirmationCheckDialog
+from gajim.gtk.util import get_icon_name
+from gajim.gtk.util import resize_window
+from gajim.gtk.util import move_window
+from gajim.gtk.util import get_app_icon_list
+from gajim.gtk.util import get_builder
+from gajim.gtk.util import set_urgency_hint
+from gajim.gtk.util import get_app_window
+from gajim.gtk.const import ControlType
 
-####################
 
-class MessageWindow:
+log = logging.getLogger('gajim.message_window')
+
+
+WINDOW_TYPES = ['never',
+                'always',
+                'always_with_roster',
+                'peracct',
+                'pertype']
+
+
+class MessageWindow(EventHelper):
     """
     Class for windows which contain message like things; chats, groupchats, etc
     """
@@ -61,6 +80,7 @@ class MessageWindow:
     ) = range(5)
 
     def __init__(self, acct, type_, parent_window=None, parent_paned=None):
+        EventHelper.__init__(self)
         # A dictionary of dictionaries
         # where _contacts[account][jid] == A MessageControl
         self._controls = {}
@@ -76,7 +96,7 @@ class MessageWindow:
         self.dont_warn_on_delete = False
 
         self.widget_name = 'message_window'
-        self.xml = gtkgui_helpers.get_gtk_builder('%s.ui' % self.widget_name)
+        self.xml = get_builder('%s.ui' % self.widget_name)
         self.window = self.xml.get_object(self.widget_name)
         self.window.set_application(app.app)
         self.notebook = self.xml.get_object('notebook')
@@ -112,18 +132,7 @@ class MessageWindow:
         id_ = self.window.connect('focus-in-event', self._on_window_focus)
         self.handlers[id_] = self.window
 
-        keys = ['<Control>f', '<Control>g', '<Control>h', '<Control>i',
-                '<Control>l', '<Control>L', '<Control><Shift>n', '<Control>u',
-                '<Control>b', '<Control>F4',
-                '<Control>w', '<Control>Page_Up', '<Control>Page_Down', '<Alt>Right',
-                '<Alt>Left', '<Alt>d', '<Alt>c', '<Alt>m', '<Alt>t', 'Escape'] + \
-                ['<Alt>'+str(i) for i in range(10)]
-        accel_group = Gtk.AccelGroup()
-        for key in keys:
-            keyval, mod = Gtk.accelerator_parse(key)
-            accel_group.connect(keyval, mod, Gtk.AccelFlags.VISIBLE,
-                self.accel_group_func)
-        self.window.add_accel_group(accel_group)
+        self._add_actions()
 
         # gtk+ doesn't make use of the motion notify on gtkwindow by default
         # so this line adds that
@@ -132,9 +141,6 @@ class MessageWindow:
         id_ = self.notebook.connect('switch-page',
             self._on_notebook_switch_page)
         self.handlers[id_] = self.notebook
-        id_ = self.notebook.connect('key-press-event',
-            self._on_notebook_key_press)
-        self.handlers[id_] = self.notebook
 
         # Tab customizations
         pref_pos = app.config.get('tabs_position')
@@ -156,16 +162,122 @@ class MessageWindow:
         self.notebook.set_show_border(app.config.get('tabs_border'))
         self.show_icon()
 
-    def change_account_name(self, old_name, new_name):
-        if old_name in self._controls:
-            self._controls[new_name] = self._controls[old_name]
-            del self._controls[old_name]
+        self.register_events([
+            ('muc-disco-update', ged.GUI1, self._on_muc_disco_update),
+        ])
 
-        for ctrl in self.controls():
-            if ctrl.account == old_name:
-                ctrl.account = new_name
-        if self.account == old_name:
-            self.account = new_name
+    def _add_actions(self):
+        actions = [
+            'change-nickname',
+            'change-subject',
+            'escape',
+            'browse-history',
+            'send-file',
+            'show-contact-info',
+            'show-emoji-chooser',
+            'clear-chat',
+            'delete-line',
+            'close-tab',
+            'move-tab-up',
+            'move-tab-down',
+            'switch-next-tab',
+            'switch-prev-tab',
+            'switch-next-unread-tab-right'
+            'switch-next-unread-tab-left',
+            'switch-tab-1',
+            'switch-tab-2',
+            'switch-tab-3',
+            'switch-tab-4',
+            'switch-tab-5',
+            'switch-tab-6',
+            'switch-tab-7',
+            'switch-tab-8',
+            'switch-tab-9',
+            'copy-text',
+        ]
+
+        disabled_for_emacs = (
+            'browse-history',
+            'send-file',
+            'close-tab'
+        )
+
+        key_theme = Gtk.Settings.get_default().get_property(
+            'gtk-key-theme-name')
+
+        for action in actions:
+            if key_theme == 'Emacs' and action in disabled_for_emacs:
+                continue
+            act = Gio.SimpleAction.new(action, None)
+            act.connect('activate', self._on_action)
+            self.window.add_action(act)
+
+    def _on_action(self, action, _param):
+        control = self.get_active_control()
+        if not control:
+            # No more control in this window
+            return
+
+        log.info('Activate action: %s, active control: %s',
+                 action.get_name(), control.contact.jid)
+
+        action = action.get_name()
+
+        # Pass the event to the control
+        res = control.delegate_action(action)
+        if res != Gdk.EVENT_PROPAGATE:
+            return res
+
+        if action == 'escape' and app.config.get('escape_key_closes'):
+            self.remove_tab(control, self.CLOSE_ESC)
+            return
+
+        if action == 'close-tab':
+            self.remove_tab(control, self.CLOSE_CTRL_KEY)
+            return
+
+        if action == 'move-tab-up':
+            old_position = self.notebook.get_current_page()
+            self.notebook.reorder_child(control.widget,
+                                        old_position - 1)
+            return
+
+        if action == 'move-tab-down':
+            old_position = self.notebook.get_current_page()
+            total_pages = self.notebook.get_n_pages()
+            if old_position == total_pages - 1:
+                self.notebook.reorder_child(control.widget, 0)
+            else:
+                self.notebook.reorder_child(control.widget,
+                                            old_position + 1)
+            return
+
+        if action == 'switch-next-tab':
+            new = self.notebook.get_current_page() + 1
+            if new >= self.notebook.get_n_pages():
+                new = 0
+            self.notebook.set_current_page(new)
+            return
+
+        if action == 'switch-prev-tab':
+            new = self.notebook.get_current_page() - 1
+            if new < 0:
+                new = self.notebook.get_n_pages() - 1
+            self.notebook.set_current_page(new)
+            return
+
+        if action == 'switch-next-unread-tab-right':
+            self.move_to_next_unread_tab(True)
+            return
+
+        if action == 'switch-next-unread-tab-left':
+            self.move_to_next_unread_tab(False)
+            return
+
+        if action.startswith('switch-tab-'):
+            number = int(action[-1])
+            self.notebook.set_current_page(number - 1)
+            return
 
     def change_jid(self, account, old_jid, new_jid):
         """
@@ -184,27 +296,37 @@ class MessageWindow:
         return sum(len(d) for d in self._controls.values())
 
     def resize(self, width, height):
-        gtkgui_helpers.resize_window(self.window, width, height)
+        resize_window(self.window, width, height)
+
+    def _on_muc_disco_update(self, event):
+        # If there is only one control in a window,
+        # the name is shown in the window title
+        if self.get_num_controls() != 1:
+            return
+        ctrl = self.get_active_control()
+        if ctrl.contact.jid != event.room_jid:
+            return
+        self.show_title()
 
     def _on_window_focus(self, widget, event):
         # on destroy() the window that was last focused gets the focus
         # again. if destroy() is called from the StartChat Dialog, this
-        # Window is not yet focused, because present() seems to be asynchron
+        # Window is not yet focused, because present() seems to be asynchronous
         # at least on KDE, and takes time.
-        if 'start_chat' in app.interface.instances:
-            if app.interface.instances['start_chat'].ready_to_destroy:
-                app.interface.instances['start_chat'].destroy()
+        start_chat = get_app_window('StartChatDialog')
+        if start_chat is not None and start_chat.ready_to_destroy:
+            start_chat.destroy()
 
         # window received focus, so if we had urgency REMOVE IT
         # NOTE: we do not have to read the message (it maybe in a bg tab)
         # to remove urgency hint so this functions does that
-        gtkgui_helpers.set_unset_urgency_hint(self.window, False)
+        set_urgency_hint(self.window, False)
 
         ctrl = self.get_active_control()
         if ctrl:
             ctrl.set_control_active(True)
             # Undo "unread" state display, etc.
-            if ctrl.type_id == message_control.TYPE_GC:
+            if ctrl.is_groupchat:
                 self.redraw_tab(ctrl, 'active')
             else:
                 # NOTE: we do not send any chatstate to preserve
@@ -224,7 +346,7 @@ class MessageWindow:
                 number_of_closed_control += 1
 
         if number_of_closed_control > 1:
-            def on_yes1(checked):
+            def _on_yes1(checked):
                 if checked:
                     app.config.set('confirm_close_multiple_tabs', False)
                 self.dont_warn_on_delete = True
@@ -239,11 +361,17 @@ class MessageWindow:
                         ctrl.minimize()
                 # destroy window
                 return False
-            YesNoDialog(
-                _('You are going to close several tabs'),
-                _('Do you really want to close them all?'),
-                checktext=_('_Do not ask me again'), on_response_yes=on_yes1,
-                transient_for=self.window)
+
+            NewConfirmationCheckDialog(
+                _('Close Tabs'),
+                _('You are about to close several tabs'),
+                _('Do you really want to close all of them?'),
+                _('_Do not ask me again'),
+                [DialogButton.make('Cancel'),
+                 DialogButton.make('Accept',
+                                   text=_('_Close'),
+                                   callback=_on_yes1)],
+                transient_for=self.window).show()
             return True
 
         def on_yes(ctrl):
@@ -281,6 +409,8 @@ class MessageWindow:
             del self.handlers[i]
         del self.handlers
 
+        self.unregister_events()
+
     def new_tab(self, control):
         fjid = control.get_full_jid()
 
@@ -296,7 +426,7 @@ class MessageWindow:
             ctrl.scroll_to_end()
 
         # Add notebook page and connect up to the tab's close button
-        xml = gtkgui_helpers.get_gtk_builder('message_window.ui', 'chat_tab_ebox')
+        xml = get_builder('message_window.ui', ['chat_tab_ebox'])
         tab_label_box = xml.get_object('chat_tab_ebox')
         widget = xml.get_object('tab_close_button')
         # this reduces the size of the button
@@ -311,7 +441,8 @@ class MessageWindow:
         id_ = tab_label_box.connect('button-press-event',
             self.on_tab_eventbox_button_press_event, control.widget)
         control.handlers[id_] = tab_label_box
-        self.notebook.append_page(control.widget, tab_label_box)
+        position = self.notebook.get_current_page() + 1
+        self.notebook.insert_page(control.widget, tab_label_box, position)
 
         self.notebook.set_tab_reorderable(control.widget, True)
 
@@ -325,7 +456,7 @@ class MessageWindow:
         # whether the tab is the active one.
         self.show_title()
         if self.get_num_controls() == 1:
-            GLib.timeout_add(500, control.msg_textview.grab_focus)
+            GLib.timeout_add(500, control.focus)
 
     def on_tab_eventbox_button_press_event(self, widget, event, child):
         if event.button == 3: # right click
@@ -337,126 +468,7 @@ class MessageWindow:
             self.remove_tab(ctrl, self.CLOSE_TAB_MIDDLE_CLICK)
         else:
             ctrl = self._widget_to_control(child)
-            GLib.idle_add(ctrl.msg_textview.grab_focus)
-
-    def accel_group_func(self, accel_group, acceleratable, keyval, modifier):
-        st = '1234567890' # alt+1 means the first tab (tab 0)
-        control = self.get_active_control()
-        if not control:
-            # No more control in this window
-            return
-
-        # CTRL mask
-        if modifier & Gdk.ModifierType.CONTROL_MASK:
-            if keyval == Gdk.KEY_h: # CTRL + h
-                if Gtk.Settings.get_default().get_property(
-                'gtk-key-theme-name') != 'Emacs':
-                    arg = GLib.Variant('s', 'none')
-                    self.window.lookup_action(
-                        'browse-history-%s' % control.control_id).activate(arg)
-                    return True
-            elif control.type_id == message_control.TYPE_CHAT and \
-            keyval == Gdk.KEY_f: # CTRL + f
-                # CTRL + f moves cursor one char forward when user uses Emacs
-                # theme
-                if not Gtk.Settings.get_default().get_property(
-                'gtk-key-theme-name') == 'Emacs':
-                    if app.interface.msg_win_mgr.mode == \
-                    app.interface.msg_win_mgr.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
-                        app.interface.roster.tree.grab_focus()
-                        return False
-                    self.window.lookup_action(
-                        'send-file-%s' % control.control_id).activate()
-                    return True
-            elif control.type_id == message_control.TYPE_CHAT and \
-            keyval == Gdk.KEY_g: # CTRL + g
-                control._on_convert_to_gc_menuitem_activate(None)
-                return True
-            elif control.type_id in (message_control.TYPE_CHAT,
-            message_control.TYPE_PM) and keyval == Gdk.KEY_i: # CTRL + i
-                self.window.lookup_action(
-                    'information-%s' % control.control_id).activate()
-                return True
-            elif keyval in (Gdk.KEY_l, Gdk.KEY_L): # CTRL + l|L
-                control.conv_textview.clear()
-                return True
-            elif keyval == Gdk.KEY_u: # CTRL + u: emacs style clear line
-                control.clear(control.msg_textview)
-                return True
-            elif control.type_id == message_control.TYPE_GC and \
-            keyval == Gdk.KEY_b: # CTRL + b
-                # CTRL + b moves cursor one char backward when user uses Emacs
-                # theme
-                if not Gtk.Settings.get_default().get_property(
-                'gtk-key-theme-name') == 'Emacs':
-                    self.window.lookup_action(
-                        'bookmark-%s' % control.control_id).activate()
-                    return True
-            # Tab switch bindings
-            elif keyval == Gdk.KEY_F4: # CTRL + F4
-                self.remove_tab(control, self.CLOSE_CTRL_KEY)
-                return True
-            elif keyval == Gdk.KEY_w: # CTRL + w
-                # CTRL + w removes latest word before cursor when User uses emacs
-                # theme
-                if not Gtk.Settings.get_default().get_property(
-                'gtk-key-theme-name') == 'Emacs':
-                    self.remove_tab(control, self.CLOSE_CTRL_KEY)
-                    return True
-            elif keyval in (Gdk.KEY_Page_Up, Gdk.KEY_Page_Down):
-                # CTRL + PageUp | PageDown
-                # Create event and send it to notebook
-                event = Gdk.Event.new(Gdk.EventType.KEY_PRESS)
-                event.window = self.window.get_window()
-                event.time = int(time.time())
-                event.state = Gdk.ModifierType.CONTROL_MASK
-                event.keyval = int(keyval)
-                self.notebook.event(event)
-                return True
-
-            if modifier & Gdk.ModifierType.SHIFT_MASK:
-                # CTRL + SHIFT
-                if control.type_id == message_control.TYPE_GC and \
-                keyval == Gdk.KEY_n: # CTRL + SHIFT + n
-                    self.window.lookup_action(
-                        'change-nick-%s' % control.control_id).activate()
-                    return True
-        # MOD1 (ALT) mask
-        elif modifier & Gdk.ModifierType.MOD1_MASK:
-            # Tab switch bindings
-            if keyval == Gdk.KEY_Right: # ALT + RIGHT
-                new = self.notebook.get_current_page() + 1
-                if new >= self.notebook.get_n_pages():
-                    new = 0
-                self.notebook.set_current_page(new)
-                return True
-
-            if keyval == Gdk.KEY_Left: # ALT + LEFT
-                new = self.notebook.get_current_page() - 1
-                if new < 0:
-                    new = self.notebook.get_n_pages() - 1
-                self.notebook.set_current_page(new)
-                return True
-
-            if chr(keyval) in st: # ALT + 1,2,3..
-                self.notebook.set_current_page(st.index(chr(keyval)))
-                return True
-
-            if keyval == Gdk.KEY_m: # ALT + M show emoticons menu
-                control.emoticons_button.get_popover().show()
-                return True
-
-            if (control.type_id == message_control.TYPE_GC and
-                    keyval == Gdk.KEY_t): # ALT + t
-                self.window.lookup_action(
-                    'change-subject-%s' % control.control_id).activate()
-                return True
-
-        # Close tab bindings
-        elif keyval == Gdk.KEY_Escape and \
-        app.config.get('escape_key_closes'): # Escape
-            self.remove_tab(control, self.CLOSE_ESC)
-            return True
+            GLib.idle_add(ctrl.focus)
 
     def _on_close_button_clicked(self, button, control):
         """
@@ -466,13 +478,11 @@ class MessageWindow:
 
     def show_icon(self):
         window_mode = app.interface.msg_win_mgr.mode
-        icon = 'org.gajim.Gajim'
         if window_mode in (MessageWindowMgr.ONE_MSG_WINDOW_PERTYPE,
                            MessageWindowMgr.ONE_MSG_WINDOW_NEVER):
             if self.type_ == 'gc':
-                icon = gtkgui_helpers.get_iconset_name_for('muc-active')
-
-        self.window.set_icon_name(icon)
+                icon = get_icon_name('muc-active')
+                self.window.set_icon_name(icon)
 
     def show_title(self, urgent=True, control=None):
         """
@@ -485,10 +495,9 @@ class MessageWindow:
             return
         unread = 0
         for ctrl in self.controls():
-            if ctrl.type_id == message_control.TYPE_GC and not \
-            app.config.get('notify_on_all_muc_messages') and not \
-            app.config.get_per('rooms', ctrl.room_jid,
-            'notify_on_all_messages') and not ctrl.attention_flag:
+            if (ctrl.is_groupchat and
+                    not ctrl.contact.can_notify() and
+                    not ctrl.attention_flag):
                 # count only pm messages
                 unread += ctrl.get_nb_unread_pm()
                 continue
@@ -502,12 +511,10 @@ class MessageWindow:
         else:
             urgent = False
 
-        if control.type_id == message_control.TYPE_GC:
-            name = control.room_jid.split('@')[0]
-            urgent = control.attention_flag or \
-                app.config.get('notify_on_all_muc_messages') or \
-                app.config.get_per('rooms', control.room_jid,
-                'notify_on_all_messages')
+        if control.is_groupchat:
+            name = control.contact.get_shown_name()
+            urgent = (control.attention_flag or
+                      control.contact.can_notify())
         else:
             name = control.contact.get_shown_name()
             if control.resource:
@@ -518,8 +525,12 @@ class MessageWindow:
             # Show the plural form since number of tabs > 1
             if self.type_ == 'chat':
                 label = Q_('?Noun:Chats')
+                if self.get_num_controls() == 1:
+                    label = name
             elif self.type_ == 'gc':
-                label = _('Groupchats')
+                label = _('Group Chats')
+                if self.get_num_controls() == 1:
+                    label = name
             else:
                 label = _('Private Chats')
         elif window_mode == MessageWindowMgr.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
@@ -537,17 +548,13 @@ class MessageWindow:
             title = title + ": " + control.account
 
         self.window.set_title(unread_str + title)
-
-        if urgent:
-            gtkgui_helpers.set_unset_urgency_hint(self.window, unread)
-        else:
-            gtkgui_helpers.set_unset_urgency_hint(self.window, False)
+        set_urgency_hint(self.window, urgent and unread > 0)
 
     def set_active_tab(self, ctrl):
         ctrl_page = self.notebook.page_num(ctrl.widget)
         self.notebook.set_current_page(ctrl_page)
         self.window.present()
-        GLib.idle_add(ctrl.msg_textview.grab_focus)
+        GLib.idle_add(ctrl.focus)
 
     def remove_tab(self, ctrl, method, reason=None, force=False):
         """
@@ -632,6 +639,7 @@ class MessageWindow:
         tab = self.notebook.get_tab_label(ctrl.widget)
         if not tab:
             return
+
         hbox = tab.get_children()[0]
         status_img = hbox.get_children()[0]
         nick_label = hbox.get_children()[1]
@@ -644,11 +652,12 @@ class MessageWindow:
             close_button.hide()
 
         # Update nick
-        nick_label.set_max_width_chars(10)
         if isinstance(ctrl, ChatControl):
             tab_label_str = ctrl.get_tab_label()
             # Set Label Color
-            gtkgui_helpers.add_css_class(nick_label, chatstate, 'gajim-state-')
+            if app.config.get('show_chatstate_in_tabs'):
+                gtkgui_helpers.add_css_class(
+                    nick_label, chatstate, 'gajim-state-')
         else:
             tab_label_str, color = ctrl.get_tab_label(chatstate)
             # Set Label Color
@@ -710,25 +719,14 @@ class MessageWindow:
     def get_origin(self):
         return self.window.get_window().get_origin()
 
-    def get_control(self, key, acct):
+    def get_control(self, jid, acct):
         """
-        Return the MessageControl for jid or n, where n is a notebook page index.
-        When key is an int index acct may be None
+        Return the MessageControl for jid
         """
-
-        if isinstance(key, str):
-            jid = key
-            try:
-                return self._controls[acct][jid]
-            except Exception:
-                return None
-        else:
-            page_num = key
-            notebook = self.notebook
-            if page_num is None:
-                page_num = notebook.get_current_page()
-            nth_child = notebook.get_nth_page(page_num)
-            return self._widget_to_control(nth_child)
+        try:
+            return self._controls[acct][jid]
+        except Exception:
+            return None
 
     def has_control(self, jid, acct):
         return acct in self._controls and jid in self._controls[acct]
@@ -779,11 +777,13 @@ class MessageWindow:
                 ind = ind - 1
                 if ind < 0:
                     ind = self.notebook.get_n_pages() - 1
-            ctrl = self.get_control(ind, None)
+
+            nth_child = self.notebook.get_nth_page(ind)
+            ctrl = self._widget_to_control(nth_child)
             if ctrl.get_nb_unread() > 0:
                 found = True
                 break # found
-            elif app.config.get('ctrl_tab_go_to_next_composing'):
+            if app.config.get('ctrl_tab_go_to_next_composing'):
                 # Search for a composing contact
                 contact = ctrl.contact
                 if first_composing_ind == -1 and contact.chatstate == 'composing':
@@ -829,51 +829,7 @@ class MessageWindow:
 
         control = self.get_active_control()
         if isinstance(control, ChatControlBase):
-            control.msg_textview.grab_focus()
-
-    def _on_notebook_key_press(self, widget, event):
-        # when tab itself is selected,
-        # make sure <- and -> are allowed for navigating between tabs
-        if event.keyval in (Gdk.KEY_Left, Gdk.KEY_Right):
-            return False
-
-        control = self.get_active_control()
-
-        if event.get_state() & Gdk.ModifierType.SHIFT_MASK:
-            # CTRL + SHIFT + TAB
-            if event.get_state() & Gdk.ModifierType.CONTROL_MASK and \
-            event.keyval == Gdk.KEY_ISO_Left_Tab:
-                self.move_to_next_unread_tab(False)
-                return True
-            # SHIFT + PAGE_[UP|DOWN]: send to conv_textview
-            if event.keyval in (Gdk.KEY_Page_Down, Gdk.KEY_Page_Up):
-                control.conv_textview.tv.event(event)
-                return True
-
-        elif event.get_state() & Gdk.ModifierType.CONTROL_MASK:
-            if event.keyval == Gdk.KEY_Tab: # CTRL + TAB
-                self.move_to_next_unread_tab(True)
-                return True
-            # Ctrl+PageUP / DOWN has to be handled by notebook
-            if event.keyval == Gdk.KEY_Page_Down:
-                self.move_to_next_unread_tab(True)
-                return True
-            if event.keyval == Gdk.KEY_Page_Up:
-                self.move_to_next_unread_tab(False)
-                return True
-
-        if event.keyval in (Gdk.KEY_Shift_L, Gdk.KEY_Shift_R,
-        Gdk.KEY_Control_L, Gdk.KEY_Control_R, Gdk.KEY_Caps_Lock,
-        Gdk.KEY_Shift_Lock, Gdk.KEY_Meta_L, Gdk.KEY_Meta_R,
-        Gdk.KEY_Alt_L, Gdk.KEY_Alt_R, Gdk.KEY_Super_L,
-        Gdk.KEY_Super_R, Gdk.KEY_Hyper_L, Gdk.KEY_Hyper_R):
-            return True
-
-        if isinstance(control, ChatControlBase):
-            # we forwarded it to message textview
-            control.msg_textview.remove_placeholder()
-            control.msg_textview.event(event)
-            control.msg_textview.grab_focus()
+            control.focus()
 
     def get_tab_at_xy(self, x, y):
         """
@@ -889,15 +845,13 @@ class MessageWindow:
             tab = self.notebook.get_tab_label(page)
             tab_alloc = tab.get_allocation()
             if horiz:
-                if (x >= tab_alloc.x) and \
-                (x <= (tab_alloc.x + tab_alloc.width)):
+                if tab_alloc.x <= x <= (tab_alloc.x + tab_alloc.width):
                     page_num = i
                     if x >= tab_alloc.x + (tab_alloc.width / 2.0):
                         to_right = True
                     break
             else:
-                if (y >= tab_alloc.y) and \
-                (y <= (tab_alloc.y + tab_alloc.height)):
+                if tab_alloc.y <= y <= (tab_alloc.y + tab_alloc.height):
                     page_num = i
 
                     if y > tab_alloc.y + (tab_alloc.height / 2.0):
@@ -928,7 +882,7 @@ class MessageWindowMgr(GObject.GObject):
             'window-delete': (GObject.SignalFlags.RUN_LAST, None, (object,)),
     }
 
-    # These constants map to common.config.opt_one_window_types indices
+    # These constants map to WINDOW_TYPES indices
     (
             ONE_MSG_WINDOW_NEVER,
             ONE_MSG_WINDOW_ALWAYS,
@@ -936,6 +890,7 @@ class MessageWindowMgr(GObject.GObject):
             ONE_MSG_WINDOW_PERACCT,
             ONE_MSG_WINDOW_PERTYPE,
     ) = range(5)
+
     # A key constant for the main window in ONE_MSG_WINDOW_ALWAYS mode
     MAIN_WIN = 'main'
     # A key constant for the main window in ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER mode
@@ -955,14 +910,12 @@ class MessageWindowMgr(GObject.GObject):
 
         # Map the mode to a int constant for frequent compares
         mode = app.config.get('one_message_window')
-        self.mode = common.config.opt_one_window_types.index(mode)
+        self.mode = WINDOW_TYPES.index(mode)
 
         self.parent_win = parent_window
         self.parent_paned = parent_paned
 
-    def change_account_name(self, old_name, new_name):
-        for win in self.windows():
-            win.change_account_name(old_name, new_name)
+        Gtk.Window.set_default_icon_list(get_app_icon_list(parent_window))
 
     def _new_window(self, acct, type_):
         parent_win = None
@@ -1009,10 +962,8 @@ class MessageWindowMgr(GObject.GObject):
             size = (app.config.get('msgwin-width'),
                     app.config.get('msgwin-height'))
             if self.mode == self.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
-                # Need to add the size of the now visible paned handle, otherwise
-                # the saved width of the message window decreases by this amount
-                handle_size = win.parent_paned.style_get_property('handle-size')
-                size = (hpaned + size[0] + handle_size, size[1])
+                # Add the hpaned position to our message window's size
+                size = (hpaned + size[0], size[1])
         elif self.mode == self.ONE_MSG_WINDOW_PERACCT:
             size = (app.config.get_per('accounts', acct, 'msgwin-width'),
                     app.config.get_per('accounts', acct, 'msgwin-height'))
@@ -1046,7 +997,7 @@ class MessageWindowMgr(GObject.GObject):
         else:
             return
 
-        gtkgui_helpers.move_window(win.window, pos[0], pos[1])
+        move_window(win.window, pos[0], pos[1])
 
     def _mode_to_key(self, contact, acct, type_, resource=None):
         if self.mode == self.ONE_MSG_WINDOW_NEVER:
@@ -1068,6 +1019,7 @@ class MessageWindowMgr(GObject.GObject):
             return type_
 
     def create_window(self, contact, acct, type_, resource=None):
+        type_ = str(type_)
         win_acct = None
         win_type = None
         win_role = None # X11 window role
@@ -1153,7 +1105,7 @@ class MessageWindowMgr(GObject.GObject):
         win = self.get_window(jid, account)
         if win:
             ctrl = win.get_control(jid, account)
-            if not ctrl.resource and ctrl.type_id != message_control.TYPE_GC:
+            if not ctrl.resource and not ctrl.is_groupchat:
                 return ctrl
         return None
 
@@ -1163,7 +1115,7 @@ class MessageWindowMgr(GObject.GObject):
         some day in the future?
         """
         ctrl = self.get_control(jid, acct)
-        if ctrl and ctrl.type_id == message_control.TYPE_GC:
+        if ctrl and ctrl.is_groupchat:
             return ctrl
         return None
 
@@ -1172,7 +1124,7 @@ class MessageWindowMgr(GObject.GObject):
         for c in self.controls():
             if acct and c.account != acct:
                 continue
-            if not type_ or c.type_id == type_:
+            if not type_ or c.type == type_:
                 ctrls.append(c)
         return ctrls
 
@@ -1223,8 +1175,10 @@ class MessageWindowMgr(GObject.GObject):
             size_width_key = type_ + '-msgwin-width'
             size_height_key = type_ + '-msgwin-height'
         elif self.mode == self.ONE_MSG_WINDOW_ALWAYS_WITH_ROSTER:
-            # Ignore any hpaned width
-            width = msg_win.notebook.get_allocation().width
+            # Ignore hpaned separator's width and calculate width ourselves
+            win_width = msg_win.window.get_allocation().width
+            hpaned_position = app.config.get('roster_hpaned_position')
+            width = win_width - hpaned_position
 
         if acct:
             app.config.set_per('accounts', acct, size_width_key, width)
@@ -1250,10 +1204,10 @@ class MessageWindowMgr(GObject.GObject):
         for w in self.windows():
             self.save_state(w)
         mode = app.config.get('one_message_window')
-        if self.mode == common.config.opt_one_window_types.index(mode):
+        if self.mode == WINDOW_TYPES.index(mode):
             # No change
             return
-        self.mode = common.config.opt_one_window_types.index(mode)
+        self.mode = WINDOW_TYPES.index(mode)
 
         controls = []
         for w in self.windows():
@@ -1283,17 +1237,16 @@ class MessageWindowMgr(GObject.GObject):
                 child = w.parent_paned.get_child2()
                 w.parent_paned.remove(child)
                 self.parent_win.lookup_action('show-roster').set_enabled(False)
-                gtkgui_helpers.resize_window(w.window,
-                        app.config.get('roster_width'),
-                        app.config.get('roster_height'))
+                resize_window(w.window,
+                              app.config.get('roster_width'),
+                              app.config.get('roster_height'))
 
         self._windows = {}
 
         for ctrl in controls:
             mw = self.get_window(ctrl.contact.jid, ctrl.account)
             if not mw:
-                mw = self.create_window(ctrl.contact, ctrl.account,
-                                        ctrl.type_id)
+                mw = self.create_window(ctrl.contact, ctrl.account, ctrl.type)
             ctrl.parent_win = mw
             ctrl.add_actions()
             ctrl.update_actions()
@@ -1305,7 +1258,7 @@ class MessageWindowMgr(GObject.GObject):
         chat_controls = {}
         for acct in app.connections:
             chat_controls[acct] = []
-        for ctrl in self.get_controls(type_=message_control.TYPE_CHAT):
+        for ctrl in self.get_controls(type_=ControlType.CHAT):
             acct = ctrl.account
             if ctrl.contact.jid not in chat_controls[acct]:
                 chat_controls[acct].append(ctrl.contact.jid)
